@@ -324,7 +324,7 @@ bundle:my_bundle  # items in this bundle
 
     if shell_completion:
         # bw generate-completions
-        help_generate_completions = _("Generates the shell completion file")
+        help_generate_completions = _("Generates the shell completion file for use with argcomplete")
         parser_generate_completions = subparsers.add_parser(
             "generate-completions",
             description=help_generate_completions,
@@ -362,7 +362,7 @@ bundle:my_bundle  # items in this bundle
     )
 
     # bw hash
-    help_hash = _("Shows a SHA1 hash that summarizes the entire configuration for this repo, node, group, or item.")
+    help_hash = _("Shows a SHA256 hash that summarizes the entire configuration for this repo, node, group, or item.")
     parser_hash = subparsers.add_parser("hash", description=help_hash, help=help_hash)
     parser_hash.set_defaults(func=bw_hash)
     parser_hash.add_argument(
@@ -465,7 +465,6 @@ bundle:my_bundle  # items in this bundle
     parser_items.add_argument(
         "-f",
         "--preview",
-        "--file-preview",  # TODO 4.0 remove
         action='store_true',
         dest='preview',
         help=_("print preview of given ITEM"),
@@ -480,29 +479,36 @@ bundle:my_bundle  # items in this bundle
         type=str,
         help=_("create DIRECTORY and fill it with rendered file previews"),
     )
-    parser_items.add_argument(
-        "--attrs",
-        action='store_true',
-        dest='show_attrs',
-        help=_("show internal item attributes"),
-    )
     parser_items.add_argument(
         "--blame",
         action='store_true',
         dest='blame',
         help=_("show information on which bundle defines each item"),
     )
-    parser_items.add_argument(
+    information_group = parser_items.add_mutually_exclusive_group()
+    information_group.add_argument(
+        "--attrs",
+        action='store_true',
+        dest='show_attrs',
+        help=_("show internal item attributes"),
+    )
+    information_group.add_argument(
         "--repr",
         action='store_true',
         dest='show_repr',
         help=_("show more verbose representation of each item"),
     )
-    parser_items.add_argument(
-        "--state",
+    information_group.add_argument(
+        "--actual-state",
         action='store_true',
-        dest='show_sdict',
-        help=_("show actual item status on node instead of should-be configuration"),
+        dest='show_actual_state',
+        help=_("show actual item status on the node instead of the expected state"),
+    )
+    parser_items.add_argument(
+        "--json",
+        action='store_true',
+        dest='format_json',
+        help=_("show output as json instead of the usual table"),
     )
 
     # bw lock
@@ -564,6 +570,12 @@ bundle:my_bundle  # items in this bundle
         nargs='+',
         type=str,
     )
+    parser_lock_add.add_argument(
+        "--skip-item-verification",
+        action='store_true',
+        dest='skip_item_verification',
+        help=_("disable verification of item selector (-i)"),
+    )
     parser_lock_add.add_argument(
         "-p",
         "--parallel-nodes",
@@ -686,7 +698,7 @@ will exit with code 47 if any matching items are locked
         nargs='*',
         type=str,
         help=_(
-            "show only partial metadata from the given key paths (e.g. `bw metadata mynode -k users/jdoe` to show `mynode.metadata['users']['jdoe']`)"),
+            "show only partial metadata from the given key paths (e.g. `bw metadata mynode -k users/jdoe` to show `mynode.metadata.get('users/jdoe')`)"),
     )
     parser_metadata.add_argument(
         "-b", "--blame",
@@ -849,13 +861,7 @@ will exit with code 47 if any matching items are locked
         dest='depends_regular',
         help=_("do not show after/needs dependencies"),
     )
-    parser_plot_subparsers_node.add_argument(
-        "--no-depends-reverse",
-        action='store_false',
-        dest='depends_reverse',
-        help=_("do not show before/needed_by dependencies"),
-    )
-
+   
     # bw plot groups-for-node
     help_plot_node_groups = _("Show where a specific node gets its groups from")
     parser_plot_subparsers_node_groups = parser_plot_subparsers.add_parser(
@@ -901,7 +907,7 @@ will exit with code 47 if any matching items are locked
         help=_(
             "request only partial metadata from the given key paths "
             "(e.g. `bw plot reactors mynode -k users/jdoe` "
-            "to show `mynode.metadata['users']['jdoe']`)"
+            "to show `mynode.metadata.get('users/jdoe')`)"
         ),
     )
     parser_plot_subparsers_node_reactors.add_argument(
