@@ -26,7 +26,8 @@ class TagFillerItem(Item):
     BUNDLE_ATTRIBUTE_NAME = "__tagfiller__"
     ITEM_TYPE_NAME = "empty_tag"
 
-    def sdict(self):
+    @property
+    def actual_state(self):
         return {}
 
 
@@ -138,8 +139,6 @@ def _add_incoming_needs(items):
 def _prepare_auto_attrs(items):
     for item in items:
         auto_attrs = item.get_auto_attrs(items)
-        # remove next line in 5.0 when killing get_auto_deps
-        auto_attrs['needs'] = set(auto_attrs.get('needs', set())) | set(item.get_auto_deps(items))
         for key, value in auto_attrs.items():
             if key not in ALLOWED_ITEM_AUTO_ATTRIBUTES:
                 raise ValueError(_("get_auto_attrs() on {item} returned illegal key {key}").format(
@@ -167,11 +166,13 @@ def _prepare_deps(items):
                         resolved_deps = tuple(resolve_selector(dep, items))
                     except NoSuchItem:
                         raise ItemDependencyError(_(
-                            "'{item}' in bundle '{bundle}' has a dependency ({dep_type}) "
-                            "on '{dep}', which doesn't exist"
+                            "'{item}' in bundle '{bundle}' on node '{node}' "
+                            "has a dependency ({dep_type}) on '{dep}', which "
+                            "doesn't exist"
                         ).format(
                             item=item.id,
                             bundle=item.bundle.name,
+                            node=item.node.name,
                             dep=dep,
                             dep_type=dep_type,
                         ))
@@ -187,7 +188,7 @@ def _prepare_deps(items):
 
 def _inject_canned_actions(items):
     """
-    Looks for canned actions like "svc_upstart:mysql:reload" in items,
+    Looks for canned actions in items,
     created actions for them and add those to the list of items.
     """
     actions = set()
@@ -195,6 +196,8 @@ def _inject_canned_actions(items):
         for canned_action_name, canned_action_attrs in item.get_canned_actions().items():
             canned_action_id = f"{item.id}:{canned_action_name}"
             canned_action_attrs.update({'triggered': True})
+            if item.canned_actions_inherit_tags:
+                canned_action_attrs.update({'tags': item.tags})
             action = Action(
                 item.bundle,
                 canned_action_id,
