@@ -3,6 +3,7 @@ from base64 import b64decode
 from collections import defaultdict
 from contextlib import contextmanager, suppress
 from datetime import datetime
+
 try:
     from functools import cache
 except ImportError:  # Python 3.8
@@ -26,13 +27,12 @@ from requests import head
 from bundlewrap.exceptions import BundleError, FaultUnavailable, TemplateError
 from bundlewrap.items import BUILTIN_ITEM_ATTRIBUTES, Item
 from bundlewrap.items.directories import validator_mode
-from bundlewrap.utils import cached_property, download, hash_local_file, sha1, tempfile
+from bundlewrap.utils import cached_property, download, hash_local_file, sha256, tempfile
 from bundlewrap.utils.remote import PathInfo
 from bundlewrap.utils.text import bold, force_text, mark_for_translation as _
 from bundlewrap.utils.text import is_subdirectory
 from bundlewrap.utils.ui import io
 
-
 DIFF_MAX_FILE_SIZE = 1024 * 1024 * 5  # bytes
 
 
@@ -210,7 +210,7 @@ def download_file(item):
                 io.debug(f"{item.node.name}:{item.bundle.name}:{item.id}: content hash is {local_hash}")
                 if local_hash != item.attributes['content_hash']:
                     raise BundleError(_(
-                        "could not download correct file from {} - sha1sum mismatch "
+                        "could not download correct file from {} - sha256sum mismatch "
                         "(expected {}, got {})"
                     ).format(
                         item.attributes['source'],
@@ -302,7 +302,7 @@ class File(Item):
         if self.attributes['content_type'] in ('binary', 'download'):
             return hash_local_file(self.template)
         else:
-            return sha1(self.content)
+            return sha256(self.content)
 
     @cached_property
     def template(self):
@@ -317,19 +317,23 @@ class File(Item):
             return data_template
         return join(self.item_dir, self.attributes['source'])
 
-    def cdict(self):
+    @property
+    def expected_state(self):
         if self.attributes['delete']:
             return None
-        cdict = {'type': 'file'}
+
+        state = {'type': 'file'}
         if self.attributes['content_type'] != 'any':
             if self.attributes['content_type'] == 'download' and self.attributes['content_hash']:
-                cdict['content_hash'] = self.attributes['content_hash']
+                state['content_hash'] = self.attributes['content_hash']
             else:
-                cdict['content_hash'] = self.content_hash
+                state['content_hash'] = self.content_hash
+
         for optional_attr in ('group', 'mode', 'owner'):
             if self.attributes[optional_attr] is not None:
-                cdict[optional_attr] = self.attributes[optional_attr]
-        return cdict
+                state[optional_attr] = self.attributes[optional_attr]
+
+        return state
 
     def fix(self, status):
         if status.must_be_created or status.must_be_deleted or 'type' in status.keys_to_fix:
@@ -338,11 +342,11 @@ class File(Item):
             for fix_type in ('content_hash', 'mode', 'owner', 'group'):
                 if fix_type in status.keys_to_fix:
                     if fix_type == 'group' and \
-                            'owner' in status.keys_to_fix:
+                        'owner' in status.keys_to_fix:
                         # owner and group are fixed with a single chown
                         continue
                     if fix_type in ('mode', 'owner', 'group') and \
-                            'content' in status.keys_to_fix:
+                        'content' in status.keys_to_fix:
                         # fixing content implies settings mode and owner/group
                         continue
                     getattr(self, "_fix_" + fix_type)(status)
@@ -387,17 +391,18 @@ class File(Item):
             group,
             quote(self.name),
         ))
+
     _fix_group = _fix_owner
 
     def _fix_type(self, status):
-        if status.sdict:
+        if status.actual_state:
             self.run("rm -rf -- {}".format(quote(self.name)))
         if not status.must_be_deleted:
             self.run("mkdir -p -- {}".format(quote(dirname(self.name))))
             self._fix_content_hash(status)
 
-    def get_auto_deps(self, items):
-        deps = []
+    def get_auto_attrs(self, items):
+        deps = set()
         for item in items:
             if item.ITEM_TYPE_NAME == 'file' and is_subdirectory(item.name, self.name):
                 raise BundleError(_(
@@ -421,7 +426,7 @@ class File(Item):
                         bundle2=item.bundle.name,
                     ))
                 else:
-                    deps.append(item.id)
+                    deps.add(item.id)
             elif item.ITEM_TYPE_NAME == 'group' and item.name == self.attributes['group']:
                 if item.attributes['delete']:
                     raise BundleError(_(
@@ -434,13 +439,20 @@ class File(Item):
                         bundle2=item.bundle.name,
                     ))
                 else:
-                    deps.append(item.id)
+                    deps.add(item.id)
+            elif item.ITEM_TYPE_NAME == "zfs_dataset":
+                if item.attributes.get('mountpoint') not in (None, "none"):
+                    if is_subdirectory(item.attributes['mountpoint'], self.name):
+                        deps.add(item.id)
             elif item.ITEM_TYPE_NAME in ('directory', 'symlink'):
                 if is_subdirectory(item.name, self.name):
-                    deps.append(item.id)
-        return deps
+                    deps.add(item.id)
+        return {
+            'needs': deps,
+        }
 
-    def sdict(self):
+    @property
+    def actual_state(self):
         use_uid = self.attributes['owner'] is not None and self.attributes['owner'].startswith('+')
         use_gid = self.attributes['group'] is not None and self.attributes['group'].startswith('+')
         path_info = PathInfo(self.node, self.name)
@@ -449,66 +461,65 @@ class File(Item):
         else:
             return {
                 'type': 'file' if path_info.is_file else path_info.stat['type'],
-                'content_hash': path_info.sha1 if path_info.is_file else None,
+                'content_hash': path_info.sha256 if path_info.is_file else None,
                 'mode': path_info.mode,
                 'owner': ('+' + path_info.owner_id) if use_uid else path_info.owner,
                 'group': ('+' + path_info.group_id) if use_gid else path_info.group,
                 'size': path_info.size,
             }
 
-    def display_on_create(self, cdict):
+    def display_on_create(self, expected_state):
         if (
             self.attributes['content_type'] not in ('any', 'base64', 'binary', 'download') and
             len(self.content) < DIFF_MAX_FILE_SIZE
         ):
-            del cdict['content_hash']
-            cdict['content'] = force_text(self.content)
+            del expected_state['content_hash']
+            expected_state['content'] = force_text(self.content)
         if self.attributes['content_type'] == 'download':
-            cdict['source'] = self.attributes['source']
-        del cdict['type']
-        return cdict
+            expected_state['source'] = self.attributes['source']
+        del expected_state['type']
+        return expected_state
 
-    def display_dicts(self, cdict, sdict, keys):
+    def display_on_fix(self, expected_state, actual_state, keys):
         if (
             'content_hash' in keys and
             self.attributes['content_type'] not in ('base64', 'binary', 'download') and
-            sdict['size'] < DIFF_MAX_FILE_SIZE and
+            actual_state['size'] < DIFF_MAX_FILE_SIZE and
             len(self.content) < DIFF_MAX_FILE_SIZE and
             PathInfo(self.node, self.name).is_text_file
         ):
             keys.remove('content_hash')
-            keys.append('content')
-            del cdict['content_hash']
-            del sdict['content_hash']
-            cdict['content'] = self.content.decode(self.attributes['encoding'])
-            sdict['content'] = get_remote_file_contents(
+            keys.add('content')
+            del expected_state['content_hash']
+            del actual_state['content_hash']
+            expected_state['content'] = self.content.decode(self.attributes['encoding'])
+            actual_state['content'] = get_remote_file_contents(
                 self.node,
                 self.name,
             ).decode('utf-8', 'backslashreplace')
         if 'type' in keys:
-            with suppress(ValueError):
-                keys.remove('content_hash')
+            keys.discard('content_hash')
         if self.attributes['content_type'] == 'download':
-            cdict['source'] = self.attributes['source']
-            sdict['source'] = ''
-        if sdict:
-            del sdict['size']
+            expected_state['source'] = self.attributes['source']
+            actual_state['source'] = ''
+        if actual_state:
+            del actual_state['size']
             if self.attributes['content_type'] == 'any':
                 with suppress(KeyError):
-                    del sdict['content_hash']
-        return (cdict, sdict, keys)
+                    del actual_state['content_hash']
+        return (expected_state, actual_state, keys)
 
-    def display_on_delete(self, sdict):
-        del sdict['content_hash']
+    def display_on_delete(self, actual_state):
+        del actual_state['content_hash']
         path_info = PathInfo(self.node, self.name)
         if (
-            sdict['size'] < DIFF_MAX_FILE_SIZE and
+            actual_state['size'] < DIFF_MAX_FILE_SIZE and
             path_info.is_text_file
         ):
-            sdict['content'] = get_remote_file_contents(self.node, self.name)
+            actual_state['content'] = get_remote_file_contents(self.node, self.name)
         if path_info.is_file:
-            sdict['size'] = f"{sdict['size']} bytes"
-        return sdict
+            actual_state['size'] = f"{actual_state['size']} bytes"
+        return actual_state
 
     def patch_attributes(self, attributes):
         if (
@@ -573,8 +584,6 @@ class File(Item):
                     exitcode, stdout = self._run_validator(cmd)
                     if exitcode == 0:
                         io.debug(f"{self.id} passed local validation")
-                    elif exitcode in (126, 127, 255):
-                        io.debug(f"{self.id} failed local validation with code {exitcode}, ignoring")
                     else:
                         raise BundleError(_(
                             "{i} failed local validation using: {c}\n\n{out}"
@@ -648,10 +657,10 @@ class File(Item):
                 ).format(item=item_id, bundle=bundle.name, source=attributes['source']))
 
         if 'encoding' in attributes and attributes.get('content_type') in (
-            'any',
-            'base64',
-            'binary',
-            'download',
+                'any',
+                'base64',
+                'binary',
+                'download',
         ):
             raise BundleError(_(
                 "content_type of {item} from bundle '{bundle}' cannot provide different encoding "
@@ -660,10 +669,10 @@ class File(Item):
 
         if (
             attributes.get('content_type', None) == "any" and (
-                'content' in attributes or
-                'encoding' in attributes or
-                'source' in attributes
-            )
+            'content' in attributes or
+            'encoding' in attributes or
+            'source' in attributes
+        )
         ):
             raise BundleError(_(
                 "{item} from bundle '{bundle}' with content_type 'any' "
