@@ -1,17 +1,11 @@
-from logging import ERROR, getLogger
 from shlex import quote
 from string import ascii_lowercase, digits
 
-from passlib.hash import md5_crypt, sha256_crypt, sha512_crypt
-
 from bundlewrap.exceptions import BundleError
 from bundlewrap.items import BUILTIN_ITEM_ATTRIBUTES, Item
 from bundlewrap.utils.crypto import bcrypt
 from bundlewrap.utils.text import force_text, mark_for_translation as _
 
-
-getLogger('passlib').setLevel(ERROR)
-
 _ATTRIBUTE_NAMES = {
     'full_name': _("full name"),
     'gid': _("GID"),
@@ -32,18 +26,6 @@ _ATTRIBUTE_OPTIONS = {
     'uid': "-u",
 }
 
-# a random static salt if users don't provide one
-# TODO #853 If we make bundlewrap.crypto a replacement for passlib, then
-# move this constant over there.
-_DEFAULT_SALT = "uJzJlYdG"
-
-HASH_METHODS = {
-    'md5': md5_crypt,
-    'sha256': sha256_crypt,
-    'sha512': sha512_crypt,
-    'bcrypt': bcrypt,
-}
-
 _USERNAME_VALID_CHARACTERS = ascii_lowercase + digits + "-_"
 
 
@@ -93,7 +75,6 @@ class User(Item):
         'full_name': None,
         'gid': None,
         'groups': None,
-        'hash_method': 'sha512',
         'home': None,
         'password': None,
         'password_hash': None,
@@ -121,21 +102,25 @@ class User(Item):
             self.attributes['delete'],
         )
 
-    def cdict(self):
+    @property
+    def expected_state(self):
         if self.attributes['delete']:
             return None
-        cdict = self.attributes.copy()
-        del cdict['delete']
-        del cdict['hash_method']
-        del cdict['password']
-        del cdict['salt']
-        del cdict['use_shadow']
-        for key in list(cdict.keys()):
-            if cdict[key] is None:
-                del cdict[key]
-        if 'groups' in cdict:
-            cdict['groups'] = set(cdict['groups'])
-        return cdict
+
+        state = self.attributes.copy()
+        del state['delete']
+        del state['password']
+        del state['salt']
+        del state['use_shadow']
+
+        for key in list(state.keys()):
+            if state[key] is None:
+                del state[key]
+
+        if 'groups' in state:
+            state['groups'] = set(state['groups'])
+
+        return state
 
     def fix(self, status):
         if self.node.os == 'freebsd':
@@ -153,7 +138,7 @@ class User(Item):
             stdin = None
             for attr, option in sorted(_ATTRIBUTE_OPTIONS.items()):
                 if (attr in status.keys_to_fix or status.must_be_created) and \
-                        self.attributes[attr] is not None:
+                    self.attributes[attr] is not None:
                     if attr == 'groups':
                         value = ",".join(self.attributes[attr])
                     elif attr == 'password_hash' and self.node.os == 'freebsd':
@@ -175,18 +160,18 @@ class User(Item):
             command += f"{self.name}"
             self.run(command, data_stdin=stdin, may_fail=True)
 
-    def display_on_create(self, cdict):
+    def display_on_create(self, expected_state):
         for attr_name, attr_display_name in _ATTRIBUTE_NAMES.items():
             if attr_name == attr_display_name:
                 # Don't change anything; the `del` below would
                 # always remove the key entirely!
                 continue
-            if attr_name in cdict:
-                cdict[attr_display_name] = cdict[attr_name]
-                del cdict[attr_name]
-        return cdict
+            if attr_name in expected_state:
+                expected_state[attr_display_name] = expected_state[attr_name]
+                del expected_state[attr_name]
+        return expected_state
 
-    def display_dicts(self, cdict, sdict, keys):
+    def display_on_fix(self, expected_state, actual_state, keys):
         for attr_name, attr_display_name in _ATTRIBUTE_NAMES.items():
             if attr_name == attr_display_name:
                 # Don't change anything; the `del`s below would
@@ -194,18 +179,18 @@ class User(Item):
                 continue
             try:
                 keys.remove(attr_name)
-            except ValueError:
+            except KeyError:
                 pass
             else:
-                keys.append(attr_display_name)
-                cdict[attr_display_name] = cdict[attr_name]
-                sdict[attr_display_name] = sdict[attr_name]
-                del cdict[attr_name]
-                del sdict[attr_name]
-        return (cdict, sdict, keys)
+                keys.add(attr_display_name)
+                expected_state[attr_display_name] = expected_state[attr_name]
+                actual_state[attr_display_name] = actual_state[attr_name]
+                del expected_state[attr_name]
+                del actual_state[attr_name]
+        return (expected_state, actual_state, keys)
 
-    def get_auto_deps(self, items):
-        deps = []
+    def get_auto_attrs(self, items):
+        deps = set()
         groups = self.attributes['groups'] or []
         for item in items:
             if item.ITEM_TYPE_NAME == "group":
@@ -226,10 +211,13 @@ class User(Item):
                         bundle2=item.bundle.name,
                     ))
                 else:
-                    deps.append(item.id)
-        return deps
+                    deps.add(item.id)
+        return {
+            'needs': deps,
+        }
 
-    def sdict(self):
+    @property
+    def actual_state(self):
         # verify content of /etc/passwd
         if self.node.os in self.node.OS_FAMILY_BSD:
             password_command = "grep -ae '^{}:' /etc/master.passwd"
@@ -258,10 +246,10 @@ class User(Item):
         else:
             entries = ('username', 'passwd_hash', 'uid', 'gid', 'gecos', 'home', 'shell')
 
-        sdict = _parse_passwd_line(passwd_grep_result.stdout_text, entries)
+        actual_state = _parse_passwd_line(passwd_grep_result.stdout_text, entries)
 
         if self.attributes['gid'] is not None and not self.attributes['gid'].isdigit():
-            sdict['gid'] = _group_name_for_gid(self.node, sdict['gid'])
+            actual_state['gid'] = _group_name_for_gid(self.node, actual_state['gid'])
 
         if self.attributes['password_hash'] is not None:
             if self.attributes['use_shadow'] and self.node.os not in self.node.OS_FAMILY_BSD:
@@ -271,43 +259,37 @@ class User(Item):
                     may_fail=True,
                 )
                 if shadow_grep_result.return_code != 0:
-                    sdict['password_hash'] = None
+                    actual_state['password_hash'] = None
                 else:
-                    sdict['password_hash'] = shadow_grep_result.stdout_text.split(":")[1]
+                    actual_state['password_hash'] = shadow_grep_result.stdout_text.split(":")[1]
             else:
-                sdict['password_hash'] = sdict['passwd_hash']
-        del sdict['passwd_hash']
+                actual_state['password_hash'] = actual_state['passwd_hash']
+        del actual_state['passwd_hash']
 
         # verify content of /etc/group
-        sdict['groups'] = set(_groups_for_user(self.node, self.name))
+        actual_state['groups'] = set(_groups_for_user(self.node, self.name))
 
-        return sdict
+        return actual_state
 
     def patch_attributes(self, attributes):
         if attributes.get('password', None) is not None:
-            # defaults aren't set yet
-            hash_method = HASH_METHODS[attributes.get(
-                'hash_method',
-                self.ITEM_ATTRIBUTES['hash_method'],
-            )]
+            # This is the default in OpenBSD.
+            #
+            # Linux PAM appears to use 5 by default. It's probably
+            # reasonable to ignore this and use 8 on Linux as well.
+            #
+            # (The Linux PAM crypt algo is configured in some file in
+            # /etc/pam.d, probably a line like "password pam_unix.so
+            # blowfish".)
+            cost = 8
+
             salt = force_text(attributes.get('salt', None))
-            if self.node.os == 'openbsd':
-                attributes['password_hash'] = bcrypt(
-                    force_text(attributes['password']),
-                    rounds=8,  # default rounds for OpenBSD accounts
-                    salt=salt,
-                )
-            elif attributes.get('hash_method') == 'md5':
-                attributes['password_hash'] = hash_method.encrypt(
-                    force_text(attributes['password']),
-                    salt=_DEFAULT_SALT if salt is None else salt,
-                )
-            else:
-                attributes['password_hash'] = hash_method.encrypt(
-                    force_text(attributes['password']),
-                    rounds=5000,  # default from glibc
-                    salt=_DEFAULT_SALT if salt is None else salt,
-                )
+
+            attributes['password_hash'] = bcrypt(
+                force_text(attributes['password']),
+                cost=cost,
+                salt=salt,
+            )
 
         if 'use_shadow' not in attributes:
             attributes['use_shadow'] = self.node.use_shadow_passwords
@@ -328,16 +310,6 @@ class User(Item):
                         "attributes besides 'delete'"
                     ).format(item=item_id, bundle=bundle.name))
 
-        if 'hash_method' in attributes and \
-                attributes['hash_method'] not in HASH_METHODS:
-            raise BundleError(
-                _("Invalid hash method for {item} in bundle '{bundle}': '{method}'").format(
-                    bundle=bundle.name,
-                    item=item_id,
-                    method=attributes['hash_method'],
-                )
-            )
-
         if 'password_hash' in attributes and (
             'password' in attributes or
             'salt' in attributes
