@@ -1,19 +1,17 @@
 from collections import defaultdict, Counter
-from collections.abc import Mapping
 from contextlib import suppress
 from os import environ
 from threading import RLock
 from traceback import TracebackException
 
-from .exceptions import MetadataPersistentKeyError
+from .exceptions import MetadataPersistentKeyError, MetadataUnavailable
 from .metadata import DoNotRunAgain
 from .node import _flatten_group_hierarchy
-from .utils import list_starts_with, randomize_order, NO_DEFAULT
+from .utils import NO_DEFAULT, error_context, list_starts_with, randomize_order
 from .utils.dicts import extra_paths_in_dict
-from .utils.ui import io, QUIT_EVENT
 from .utils.metastack import Metastack
 from .utils.text import bold, mark_for_translation as _, red
-
+from .utils.ui import io, QUIT_EVENT
 
 MAX_METADATA_ITERATIONS = int(environ.get("BW_MAX_METADATA_ITERATIONS", "1000"))
 
@@ -101,31 +99,13 @@ class PathSet:
             return result
 
 
-class NodeMetadataProxy(Mapping):
+class NodeMetadataProxy:
     def __init__(self, metagen, node):
         self._metagen = metagen
         self._node = node
         self._completed_paths = PathSet()
         self._metastack = Metastack()
 
-    def __contains__(self, key):
-        try:
-            self.get(key, _backwards_compatibility_default=False)
-        except KeyError:
-            return False
-        else:
-            return True
-
-    def __getitem__(self, key):
-        return self.get((key,), _backwards_compatibility_default=False)
-
-    def __iter__(self):
-        for key, value in self.get(tuple()).items():
-            yield key, value
-
-    def __len__(self):
-        return len(self.keys())
-
     @property
     def blame(self):
         if self._metagen._in_a_reactor:
@@ -140,16 +120,7 @@ class NodeMetadataProxy(Mapping):
         else:
             return self._metastack
 
-    def get(self, path, default=NO_DEFAULT, _backwards_compatibility_default=True):
-        if (
-            default == NO_DEFAULT and
-            _backwards_compatibility_default and
-            not self._metagen._in_a_reactor and
-            "/" not in path
-        ):
-            # make node.metadata.get('foo') work as if it was still a dict
-            # TODO remove in 5.0
-            default = None
+    def get(self, path, default=NO_DEFAULT):
         if not isinstance(path, (tuple, list)):
             path = tuple(path.split("/"))
 
@@ -191,7 +162,7 @@ class NodeMetadataProxy(Mapping):
 
             try:
                 return self._metastack.get(path)
-            except KeyError as exc:
+            except MetadataUnavailable as exc:
                 if default != NO_DEFAULT:
                     return default
                 else:
@@ -200,15 +171,6 @@ class NodeMetadataProxy(Mapping):
                             ((self._node.name, path), exc)
                     raise exc
 
-    def items(self):
-        return self.get(tuple()).items()
-
-    def keys(self):
-        return self.get(tuple()).keys()
-
-    def values(self):
-        return self.get(tuple()).values()
-
 
 class MetadataGenerator:
     def __init__(self):
@@ -224,7 +186,7 @@ class MetadataGenerator:
         self._reactors = {}
         # which reactors are currently triggered (and by what)
         self._reactors_triggered = defaultdict(set)
-        # which reactors raised a KeyError (and for what)
+        # which reactors raised a MetadataUnavailable (and for what)
         self._reactors_with_keyerrors = {}
         # maps provided paths to their reactors
         self._provides_tree = ReactorTree()
@@ -262,15 +224,15 @@ class MetadataGenerator:
                 break
             elif only_keyerrors:
                 if set(self._reactors_triggered.keys()).difference(reactors_run):
-                    io.debug("all reactors raised KeyErrors, but new ones were triggered")
+                    io.debug("all reactors raised MetadataUnavailable, but new ones were triggered")
                 else:
-                    io.debug("reactor run completed, all threw KeyErrors")
+                    io.debug("reactor run completed, all threw MetadataUnavailable")
                     break
             io.debug("reactor run completed, rerunning relevant reactors")
 
         if self._reactors_with_keyerrors:
             msg = _(
-                "These metadata reactors raised a KeyError "
+                "These metadata reactors raised MetadataUnavailable "
                 "even after all other reactors were done:"
             )
             for source, path_exc in sorted(self._reactors_with_keyerrors.items()):
@@ -372,7 +334,7 @@ class MetadataGenerator:
             yield (
                 reactor_id,
                 f"running reactor {reactor_id} because "
-                f"it previously raised a KeyError for: {path_exc[0]}"
+                f"it previously raised MetadataUnavailable for: {path_exc[0]}"
             )
 
     def __run_reactors(self):
@@ -404,7 +366,7 @@ class MetadataGenerator:
 
         return reactors_run, only_keyerrors
 
-    def __run_reactor(self, node, reactor_name, reactor):
+    def __run_reactor(self, node, reactor_name, reactor):  # skipcq: PY-R1000
         # make sure the reactor doesn't react to its own output
         old_metadata = node.metadata._metastack.pop_layer(1, reactor_name)
         self._in_a_reactor = True
@@ -414,27 +376,13 @@ class MetadataGenerator:
         self._reactor_runs[self._current_reactor] += 1
         try:
             new_metadata = reactor(node.metadata)
-        except KeyError as exc:
+        except MetadataUnavailable as exc:
             if self._current_reactor not in self._reactors_with_keyerrors:
-                # Uncomment this in 5.0 and remove the rest of this block
-                # # this is a KeyError that didn't result from metadata.get()
-                # io.stderr(_(
-                #     "{x} KeyError while executing metadata reactor "
-                #     "{metaproc} for node {node}:"
-                # ).format(
-                #     x=red("!!!"),
-                #     metaproc=reactor_name,
-                #     node=node.name,
-                # ))
-                # raise exc
                 self._reactors_with_keyerrors[self._current_reactor] = (
-                    ('UNKNOWN', ('UNKNOWN',)),
+                    (node.name, exc.path),
                     exc,
                 )
-            io.debug(
-                f"{self._current_reactor} raised KeyError: "
-                f"{self._reactors_with_keyerrors[self._current_reactor]}"
-            )
+                io.debug(f"{self._current_reactor} raised MetadataUnavailable for {exc.path}")
             return False
         except DoNotRunAgain:
             self._reactors[self._current_reactor]['raised_donotrunagain'] = True
@@ -475,6 +423,15 @@ class MetadataGenerator:
                 reactor_name=reactor_name,
             ))
 
+        if len(self._reactors[self._current_reactor]['requested_paths']) == 0:
+            raise ValueError(_(
+                "{reactor_name} on {node_name} did not request any "
+                "metadata, you might want to use defaults instead"
+            ).format(
+                node_name=node.name,
+                reactor_name=reactor_name,
+            ))
+
         if self._verify_reactor_provides and getattr(reactor, '_provides', None):
             extra_paths = extra_paths_in_dict(new_metadata, reactor._provides)
             if extra_paths:
@@ -489,11 +446,12 @@ class MetadataGenerator:
                 ))
 
         try:
-            node.metadata._metastack.set_layer(
-                1,
-                reactor_name,
-                new_metadata,
-            )
+            with error_context(reactor=reactor_name, node=node.name):
+                node.metadata._metastack.set_layer(
+                    1,
+                    reactor_name,
+                    new_metadata,
+                )
         except TypeError as exc:
             # TODO catch validation errors better
             io.stderr(_(
