@@ -39,7 +39,7 @@ from .utils.dicts import (
     dict_to_text,
     dict_to_toml,
     diff_dict,
-    hash_statedict,
+    hash_state_dict,
     set_key_at_path,
     normalize_dict,
     validate_dict,
@@ -600,10 +600,10 @@ class Node:
                     ))
 
     @cached_property
-    def cdict(self):
+    def expected_state(self):
         node_dict = {}
         for item in self.items:
-            if item.ITEM_TYPE_NAME != 'action':  # actions have no cdict
+            if item.ITEM_TYPE_NAME != 'action':  # actions have no expected_state
                 node_dict[item.id] = item.hash()
         return node_dict
 
@@ -626,7 +626,7 @@ class Node:
         self._attributes = convert_magic_strings(self.repo, self._attributes)
 
     def group_membership_hash(self):
-        return hash_statedict(sorted(names(self.groups)))
+        return hash_state_dict(sorted(names(self.groups)))
 
     @property
     def immediate_groups(self):
@@ -675,7 +675,7 @@ class Node:
         return False
 
     def hash(self):
-        return hash_statedict(self.cdict)
+        return hash_state_dict(self.expected_state)
 
     def in_any_group(self, group_list):
         for group_name in group_list:
@@ -763,8 +763,8 @@ class Node:
 
         try:
             self.repo.hooks.node_apply_start(
-                self.repo,
-                self,
+                repo=self.repo,
+                node=self,
                 interactive=interactive,
             )
         except SkipNode as exc:
@@ -828,8 +828,8 @@ class Node:
         ))
 
         self.repo.hooks.node_apply_end(
-            self.repo,
-            self,
+            repo=self.repo,
+            node=self,
             duration=result.duration,
             interactive=interactive,
             result=result,
@@ -858,10 +858,6 @@ class Node:
     def metadata(self):
         return self.repo._metadata_proxy_for_node(self.name)
 
-    def metadata_get(self, *args, **kwargs):
-        # TODO remove in 5.0
-        return self.metadata.get(*args, _backwards_compatibility_default=False, **kwargs)
-
     def metadata_hash(self):
         return hash_metadata(self.metadata)
 
@@ -886,13 +882,6 @@ class Node:
                     reactor,
                 )
 
-    @property
-    def partial_metadata(self):
-        """
-        Deprecated, remove in 5.0.0
-        """
-        return self.metadata
-
     def rename(self, new_name):
         if not self.is_toml:
             raise ValueError(_(
@@ -924,7 +913,10 @@ class Node:
             # has completed on the node before trying to reuse the
             # multiplexed connection.
             if self._ssh_first_conn_lock.acquire(False):
-                self.repo.hooks.node_ssh_connect(self.repo, self)
+                self.repo.hooks.node_ssh_connect(
+                    repo=self.repo,
+                    node=self,
+                )
                 try:
                     with io.job(_("{}  establishing connection...").format(bold(self.name))):
                         operations.run(
