@@ -395,6 +395,7 @@ def run_ipmitool(hostname, username, password, command, interface=None, log_func
 
     return result
 
+
 def run_routeros(hostname, username, password, *args):
     with ROUTEROS_CONNECTIONS_LOCK:
         try:
@@ -473,7 +474,7 @@ def upload(
         "-o",
         "StrictHostKeyChecking=no" if add_host_keys else "StrictHostKeyChecking=yes",
     ]
-    extra_args = environ.get("BW_SCP_ARGS", environ.get("BW_SSH_ARGS", "")).strip()
+    extra_args = environ.get("BW_SCP_ARGS", '').strip()
     if extra_args:
         scp_command.extend(split(extra_args))
     scp_command.append(local_path)
