@@ -5,8 +5,6 @@ from os import listdir, mkdir, walk
 from os.path import abspath, dirname, isdir, isfile, join
 from sys import version_info
 
-from .utils.node_lambda import parallel_node_eval
-
 try:
     from tomllib import loads as toml_load
 except ImportError:
@@ -24,6 +22,7 @@ from .bundle import FILENAME_ITEMS
 from .exceptions import (
     NoSuchGroup,
     NoSuchNode,
+    NoSuchTarget,
     NoSuchRepository,
     MissingRepoDependency,
     RepositoryError,
@@ -38,8 +37,9 @@ from .utils import (
     get_file_contents,
     names,
 )
+from .utils.dicts import hash_state_dict
 from .utils.scm import get_git_branch, get_git_clean, get_rev
-from .utils.dicts import hash_statedict
+from .utils.node_lambda import parallel_node_eval
 from .utils.text import bold, mark_for_translation as _, red, validate_name
 from .utils.ui import io
 
@@ -193,13 +193,21 @@ class HooksProxy:
                     files.append(filename)
 
             # define a function that calls all hook functions
-            def hook(*args, **kwargs):
+            def hook(**kwargs):
+                level_hint = _("repo-level hooks")
+                if 'node' in kwargs:
+                    level_hint = _("node-level hooks for node {}").format(kwargs['node'].name)
+                elif 'nodes' in kwargs:
+                    level_hint += _(" for {} nodes").format(len(kwargs['nodes']))
+
                 for filename in files:
-                    with io.job(_("{event}  Running hooks from {filename}").format(
+                    with io.job(_("{event}  Running {level_hint} from {filename}").format(
                         event=bold(event),
+                        level_hint=level_hint,
                         filename=filename,
                     )):
-                        self.__module_cache[filename][event](*args, **kwargs)
+                        with error_context(filename=filename):
+                            self.__module_cache[filename][event](**kwargs)
             self.__hook_cache[event] = hook
 
         return self.__hook_cache[event]
@@ -297,8 +305,9 @@ class Repository(MetadataGenerator):
         """
         try:
             assert isdir(path)
-            assert isfile(join(path, "nodes.py"))
-            assert isfile(join(path, "groups.py"))
+            assert isdir(join(path, DIRNAME_BUNDLES))
+            assert isfile(join(path, FILENAME_NODES))
+            assert isfile(join(path, FILENAME_GROUPS))
         except AssertionError:
             return False
         return True
@@ -332,7 +341,7 @@ class Repository(MetadataGenerator):
         return get_git_branch()
 
     @cached_property
-    def cdict(self):
+    def expected_state(self):
         repo_dict = {}
         for node in self.nodes:
             repo_dict[node.name] = node.hash()
@@ -538,20 +547,18 @@ class Repository(MetadataGenerator):
             raise NoSuchNode(node_name)
 
     def group_membership_hash(self):
-        return hash_statedict(sorted(names(self.groups)))
+        return hash_state_dict(sorted(names(self.groups)))
 
     @property
     def groups(self):
-        # TODO 5.0 make this a cached set
-        return sorted(self.group_dict.values())
+        return set(self.group_dict.values())
 
     def hash(self):
-        return hash_statedict(self.cdict)
+        return hash_state_dict(self.expected_state)
 
     @property
     def nodes(self):
-        # TODO 5.0 make this a cached set
-        return sorted(self.node_dict.values())
+        return set(self.node_dict.values())
 
     def nodes_in_all_groups(self, group_names):
         """
@@ -670,7 +677,7 @@ class Repository(MetadataGenerator):
         !bundle:my_bundle  # all nodes without this bundle
         !group:my_group    # all nodes not in this group
         "lambda:node.metadata_get('foo/magic', 47) < 3"
-        # all nodes whose metadata["foo"]["magic"] is less than three
+        # all nodes whose metadata.get("foo/magic") is less than three
 
         :param lambda_workers: number of parallel workers to check lambda condition on nodes
         :return list of nodes matching any of the given target-strings
@@ -689,7 +696,7 @@ class Repository(MetadataGenerator):
                 targets.update(self.nodes_without_bundle(bundle_name))
             elif name.startswith("!group:"):
                 group_name = name.split(":", 1)[1]
-                targets.update(self.nodes_in_group(group_name))
+                targets.update(self.nodes_not_in_group(group_name))
             elif name.startswith("lambda:"):
                 lambda_str = name.split(":", 1)[1]
                 targets.update(self.nodes_matching_lambda(lambda_str, lambda_workers))
@@ -701,7 +708,7 @@ class Repository(MetadataGenerator):
                         group = self.get_group(name)
                         targets.update(group.nodes)
                     except NoSuchGroup:
-                        continue
+                        raise NoSuchTarget(name)
 
         return list(targets)
 
@@ -709,7 +716,7 @@ class Repository(MetadataGenerator):
         repo_dict = {}
         for node in self.nodes:
             repo_dict[node.name] = node.metadata_hash()
-        return hash_statedict(repo_dict)
+        return hash_state_dict(repo_dict)
 
     def populate_from_path(self, path):
         if not self.is_repo(path):
